;;; **********************************************************************
;;; Copyright (c) 1995 Heinrich Taube and Tobias Kunze.  All rights
;;; reserved. Use and copying of this software and preparation of
;;; derivative works based upon this software are permitted and may be
;;; copied as long as no fees or compensation are charged for use, 
;;; copying, or accessing this software and all copies of this software
;;; include this copyright notice.
;;; **********************************************************************

(in-package :stella)

(eval-when (compile load eval)
  (ccl:require :quickdraw)
  (ccl:require :scrollers "ccl:library;scrollers")
  (ccl:require :resources "ccl:library;resources")
  #-ccl-3
  (ccl:require :scrolling-fred-dialog-item 
               "ccl:library;scrolling-fred-dialog-item")
  (ccl:require :text-edit-dialog-item                   ; layout.lisp
               "ccl:examples;text-edit-dialog-item")
  )

;;;
;;; ccl-3 patches...
;;;

#+ccl-3
(let ((ccl:*warn-if-redefine* nil))
  ; ccl:listener is no more of type ccl:fred-mixin although it is a
  ; ccl:fred-window  :(
  ; patch until help from digitool
  
  (defmethod ccl:set-window-package ((w ccl:fred-window) pkg)
    (ccl:set-window-package (ccl:fred-item w) pkg))
  
  ;(defmethod ccl:display-in-windows-menu ((w ccl:window))
  ;  (or (ccl:window-shown-p w)
  ;      (ccl:view-get w :display-in-menu-when-hidden t)))
  
  ;; this text-edit-dialog-item had bogus debug print 
  #-powerpc
  (defmethod ccl::view-draw-contents :after ((item ccl::text-edit-dialog-item)
                                             &aux wp te size)
    (let ((my-dialog (ccl::view-window item))
          (item-position (ccl::view-position item))
          (item-size (ccl::view-size item))
          (enabled-p (ccl::dialog-item-enabled-p item))
          (colorp (ccl::color-or-gray-p item)))
      (when (ccl::installed-item-p item)
        (ccl::with-slot-values (ccl::dialog-item-handle ccl::text-justification)
                               item
          (ccl::without-interrupts
           (ccl::rlet ((rect :rect))
             (ccl::rset rect rect.topleft item-position)
             (ccl::rset rect rect.bottomright
                        (ccl::add-points item-position item-size))
             (setq te (ccl::dialog-te-handle my-dialog))
             (setq size (#_GetHandleSize ccl::dialog-item-handle))           
             (ccl::with-fore-color (if (and colorp (not enabled-p))
                                     ccl::*gray-color* 
                                     (ccl::part-color item :text))
               (ccl::with-back-color (ccl::part-color item :body)
                 (if (eq item (ccl::current-key-handler my-dialog))
                   (progn
                     (ccl::%hput-long te (ccl::%get-long (setq wp (ccl::wptr my-dialog)) 68) 74)
                     (ccl::%hput-long te (ccl::%get-long wp 72) 78)
                     ; (#_EraseRect rect)
                     (#_TEUpdate rect te))
                   (ccl::with-pointers ((tp ccl::dialog-item-handle))
                     (#_TextBox tp size rect ccl::text-justification))))))))))))

;;;
;;; point stuff
;;;

(defmacro xy (x y) 
  (if (and (constantp x) (constantp y))
    (ccl:make-point x y)
    `(ccl:make-point ,x ,y)))

(defmacro xy-x (xy) `(ccl:point-h ,xy))
(defmacro xy-y (xy) `(ccl:point-v ,xy))
(defmacro xy+ (xy1 xy2) `(ccl:add-points ,xy1 ,xy2))
(defmacro xy- (xy1 xy2) `(ccl:subtract-points ,xy1 ,xy2))
(defmacro xyinc (xy dx dy)
  `(ccl:add-points ,xy (ccl:make-point ,dx ,dy)))
(defmacro xy= (xy1 xy2) `(= ,xy1 ,xy2))

;;;
;;; cursor stuff
;;;

(defun cross-hair-cursor () (#_GetResource :curs 2))

;;;
;;; font stuff
;;;

(defun font-name (font)
  (first font))

(defun font-size (font)
  (second font))

(defun font-mode (font)
  (third font))

(defun font-face (font)
  (fourth font))

(defun string-extent (string font &optional extra)
  (let ((w (ccl:string-width string font))
        (h (font-height font)))
    (if extra (xy+ (xy w h) extra) (xy w h))))

(defun font-height (&optional font)
  ;; why dont they define this?
  (multiple-value-bind (a d m l) (ccl:font-info font)
    (values (+ a d l) a d l m)))

(defun bold-face (font)
  (setf font (copy-list font))
  (setf (fourth font) ':bold)
  font)

(defun italic-face (font)
  (setf font (copy-list font))
  (setf (car (last font)) ':italic)
  font)

(defun make-font (&key name size face mode
                       (defaults '( "Geneva" 10 :plain :srcor)))
  (let ((new (or (copy-list defaults) (make-list 4))))
    (when name (setf (first new) name))
    (when size (setf (second new) size))
    (when mode (setf (third new) mode))
    (when face (setf (fourth new) face))
    new))

(defmacro view-width (view)
  `(ccl:point-h (ccl:view-size ,view)))

(defmacro view-height (view)
  `(ccl:point-v (ccl:view-size ,view)))

(defmacro view-left (view)
  `(ccl:point-h (ccl:view-position ,view)))

(defmacro view-top (view)
  `(ccl:point-v (ccl:view-position ,view)))

(defun view-bottom (view)
  (+ (xy-y (ccl:view-position view))
     (xy-y (ccl:view-size view))))

(defun view-right (view)
  (+ (xy-x (ccl:view-position view))
     (xy-x (ccl:view-size view))))

(defmacro find-subview (view name)
  `(ccl:view-named ,name ,view))


(defun find-subview-if (predicate view &rest args)
  (apply #'find-if predicate (ccl:view-subviews view) args))

(defun set-window-grow (w &optional min max)
  (let ((r (ccl:window-grow-rect w)))
    (when min 
      (ccl:rset r :rect.topleft min))
    (when max 
      (ccl:rset r :rect.bottomright (xy-x min)))))

;;;
;;;
;;;

(defun pop-up-menu-select (menu item)
  ;; make item be the default selection in popup. item 
  ;; can be an item or title.
  (let* ((items (ccl:menu-items menu))
         (place (position item items 
                          :key (if (stringp item)
                                 #'ccl:menu-item-title
                                 #'identity)
                          :test (if (stringp item) 
                                  #'string-equal
                                  #'eq))))
    (unless (= (ccl::pop-up-menu-default-item menu)
               (1+ place))
      (setf (ccl::pop-up-menu-default-item menu)
            (1+ place)))
    (ccl:invalidate-view menu)
    (ccl:menu-item-action (elt items place))))


;;; 
;;; this is a sys7 trap...

#-powerpc
(traps::deftrap traps::_PlotIconID ((traps::Rect :rect) 
                                    (traps::align :short) 
                                    (traps::transform :short)
                                    (traps::ResID :short))
                (:stack :signed-integer)
  (:stack-trap #xABC9 :d0 #x0500 traps::Rect traps::align 
               traps::transform traps::ResID))



